function Data=Mast(C,N,MOD,ENC)
% MAST: Master for computing pi using MPITB
%
%	Data = Mast ( C, N, MOD, ENC )
%
%  C	#computers used (NOT including this one)
%	If C= 0, sequential algorithm on _this_ computer
%	If C=-1, sequential algorithm on 1st slave
%  N	#subdivisions of the [0, 1] interval
%  MOD	communication primitive:  (s)end (r)educe
%  ENC	routing option: (d)irect (n)o
%
%  Data	structure contains:
%	pi	estimated pi value
%	err	error
%	time	from argument xmit to pi computed
%
% pi is obtained by numeric integration of arctg'(x) = 1/(1+x^2) between 0..1
% result is pi/4, multiply then by 4
% MATLAB has a pi function, that we use to estimate error
%
% lamboot should have booted at least C+1 computers
% You can MPI_Init: use a matching ENC / TROLLIUSRTF (RTF_MPIC2C)
% - if TROLLIUSRTF doesn't match, run is aborted
% - if you don't MPI_Init, this function will lamboot with a matching ENC
% You can run both modes (s,r) for same ENC (d?) using "Todos"
% For the other ENC (n?), exit MATLAB, enter again, use "Todos" again
% "Todos" will lamboot then with the other ENC
%

Data=[]; TAG=7; DEBUG=0;
			MastFLG=0;	% Will we run Mast(0) in 1st slave?
if nargin>1 & C<0, C=1;	MastFLG=1; end	% return result using disk save/load
%%%%%%%%%%
% ArgChk %
%%%%%%%%%%
if nargin<4, help(mfilename), return, end
flag=0;					CC=int2str(C);
flag=flag | fix(C)~=C | C<0;		NN=int2str(N);
flag=flag | fix(N)~=N | N<1;		MOD=lower(MOD);
flag=flag | isempty(findstr(MOD,'sr'));	ENC=lower(ENC);
flag=flag | isempty(findstr(ENC,'dn'));
if flag, help(mfilename), return, end

if C
%%%%%%%%%%%%%%%%%%%
% Test daemon LAM % # hosts > C
%%%%%%%%%%%%%%%%%%%
  [info flag]=MPI_Initialized;
  if info | ~flag
    RTFLAGS=RTF_HOMOG+RTF_MPIC2C*(ENC=='d');
    putenv(['TROLLIUSRTF=' int2str(RTFLAGS)]), MPI_Init;
  end
  RTFLAGS=str2num(getenv('TROLLIUSRTF'));
  if ~bitand(RTFLAGS,RTF_HOMOG)
	warning('use mpirun -O for homogeneous cluster!!!');
  end
  if ~bitand(RTFLAGS,RTF_MPIC2C)
    if ENC~='n', error('ENC parameter does not match'); end
  else
    if ENC~='d', error('ENC parameter does not match'); end
  end
  [info attr flag]=MPI_Attr_get(MPI_COMM_WORLD,MPI_UNIVERSE_SIZE);
  if info | ~flag, error('attribute MPI_UNIVERSE_SIZE does not exist?'), end
  if attr<=C, error(['need ' int2str(C+1) ' computers in LAM']), end

%%%%%%%%%
% SPAWN %
%%%%%%%%%
  if DEBUG
    DISP=getenv('DISPLAY');
    [info children errs]=MPI_Comm_spawn('/usr/X11R6/bin/xterm',...
		{'-display' DISP '-e' 'matlab'},C,'NULL',0,'SELF');
  else
    [info children errs]=MPI_Comm_spawn('matlab',...
				  {'-nosplash'},C,'NULL',0,'SELF');
  end
  if info | any(errs),	error('cannot start MATLABs'), end
  [info numt] = MPI_Comm_remote_size(children);
  if info | numt~=C,	error('cannot start MATLABs'), end
  [info NEWORLD]=MPI_Intercomm_merge(children,0);
  if info,		error('cannot start MATLABs'), end

  if DEBUG
    MPI_Errhandler_set(NEWORLD,MPI_ERRORS_RETURN);
  end

%%%%%%%%%%%
% COMMAND %
%%%%%%%%%%%
  if MastFLG				% Running Mast(0) in 1st slave
    FN=[ pwd '/tmp.mat'];
    cmd=['Data=Mast(0,' NN ',''' MOD ''',''' ENC '''); save ' FN ' Data'];
    MPI_Bcast(cmd,0,NEWORLD);		% Pass command
    while ~exist(FN,'file'), pause(1), end
    load(FN), delete(FN)
  else

%%%%%%%%%%%%
% PARALLEL %
%%%%%%%%%%%%
    cmd=['Work(' CC ',' NN ',''' MOD ''')'];
    MPI_Bcast(cmd,0,NEWORLD);		% Pass command
%   MPI_Barrier(NEWORLD);		% initial Synch
    NULL=[];
    for numt=1:C,  MPI_Recv(NULL,MPI_ANY_SOURCE,TAG,NEWORLD); end
    for numt=1:C,  MPI_Send(NULL,numt,          TAG,NEWORLD); end
%%%%%%%%%
    tic %
%%%%%%%%%
    P2=0; Psum=0;
    switch(MOD)
    case 'r' %%%%%%%%%%%%%
keyboard
	MPI_Reduce(P2,Psum,'SUM',0,NEWORLD);
    case 's' %%%%%%%%%%%%%
    for numt=1:C,  MPI_Recv(P2,MPI_ANY_SOURCE,TAG,NEWORLD); Psum=Psum+P2; end
    end

    Psum     =Psum/N;		% better done in master: don't lose resolution
    Data.pi  =Psum;
    Data.err =Psum-pi;
%%%%%%%%%%%%%%%%%%%
    Data.time=toc;%
%%%%%%%%%%%%%%%%%%%
%   MPI_Finalize;		% No, no, perhaps we want to go on
  end  % MastFLG

else % if C
%%%%%%%%%%%%%%
% SEQUENTIAL %
%%%%%%%%%%%%%%
  tic %
%%%%%%%
  width=1/N; Sum=0;
  i=0:N-1;		% for i=0:N-1
  x=(i+0.5)*width;	%   x=(i+0.5)*width;
  Sum=sum(4./(1+x.^2));	%   Sum=Sum+4/(1+x^2);
			% end
  Sum      =Sum*width;
  Data.pi  =Sum;
  Data.err =Sum-pi;
%%%%%%%%%%%%%%%%%
  Data.time=toc;%
%%%%%%%%%%%%%%%%%

end % if C

if abs(Data.err)>5E-10, warning('pretty nice pi value!'), end

